/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks: SIN <sin@sapporoworks.ne.jp>
            http://homepage2.nifty.com/spw/
    He told me usage of RAW socket on Win32 !

 *****************************************************************************/

#include <ctype.h>
#include <string.h>
#include "error.h"
#include "depend.h"
#include "promisc.h"

/*============================================================================
 Set network interface to promiscus mode.
 [INPUT]
  nit_dev   : NIT device name
  nic_name  : Name of network interface  
  chunksize     : Chunk size
 [RETURN]
  ERR_PROMISC_OPEN      : Can not open device 
  ERR_PROMISC_READ      : Can not read status 
  ERR_PROMISC_WRITE         : Can not write status 
  ERR_PROMISC_BIND      : Can not bind (IRIX only)
  ERR_PROMISC_SETSOCKOPT    : setsockopt failure (IRIX only) 
  ERR_PROMISC_TOOLONG_NITDEV    : Too long nit_dev
  ERR_PROMISC_TOOLONG_NICNAME   : Too long nic_name
  otherwise         : device handle
  ============================================================================
*/
SOCKET setnic_promisc(char *nit_dev,char *nic_name,int *chunksize)
{
    WSADATA             WSAData;
    SOCKET              sock;
    unsigned int        Optval=1;
    int                 nic_num;
    char                buf[4096],*qnic_name;
    DWORD               bBytesReturned;
    SOCKET_ADDRESS_LIST *salist;
    SOCKADDR_IN         addr,*nic_addr;
    
    WSAStartup(MAKEWORD(2,2),&WSAData);
     
    /* Create RAW socket */
    sock=WSASocket(AF_INET,SOCK_RAW,IPPROTO_IP,NULL,0,WSA_FLAG_OVERLAPPED);
    if(sock==INVALID_SOCKET)
        return(ERR_PROMISC_OPEN);
     
    /* enum adapter list */
    if(WSAIoctl(sock,SIO_ADDRESS_LIST_QUERY,NULL,0,buf,4096,
                &bBytesReturned,NULL,NULL)==SOCKET_ERROR)
        return(ERR_PROMISC_READ);   
    salist = (SOCKET_ADDRESS_LIST *)buf;

    /* search specified nic_num */
    for (nic_num=0;nic_num<salist->iAddressCount;nic_num++){
        nic_addr  =(SOCKADDR_IN *)salist->Address[nic_num].lpSockaddr;
        qnic_name =inet_ntoa(nic_addr->sin_addr);
        if (!strcmp(qnic_name,nic_name)) break;
    }
    if (nic_num==salist->iAddressCount){
        nic_num  =0;
        nic_addr =(SOCKADDR_IN *)salist->Address[nic_num].lpSockaddr;
        qnic_name=inet_ntoa(nic_addr->sin_addr);
        errorlog(ERRLOG_WARNING,"specified interface %s is not found\n",
            nic_name);
        errorlog(ERRLOG_WARNING,"'%s' is selected.\n",qnic_name);
    }

    /* Bind */
    addr.sin_addr.s_addr=nic_addr->sin_addr.s_addr;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(0);
    if (bind(sock,(SOCKADDR *)&addr,sizeof(addr))==SOCKET_ERROR)
        return(ERR_PROMISC_BIND);
 
    /* Set NIC to promiscus mode */
    Optval=1;
    if(WSAIoctl(sock,SIO_RCVALL,&Optval,sizeof(Optval),NULL,0,
                &bBytesReturned,NULL,NULL)==SOCKET_ERROR)
        return(ERR_PROMISC_WRITE);

    return(sock);
}
