/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 *****************************************************************************/

#include <unistd.h>
#include <string.h>
#include "depend.h"
#include "param.h"
#include "promisc.h"
#include "analyze.h"
#include "gddconfig.h"
#include "error.h"

#include "analyze.h"

/*============================================================================
 Main loop (read packet and analyze it)
 [INPUT]
  gddc          : Configuration data of gdd
 [RETURN]
  none
  ============================================================================
*/
void sniffloop(GDDCONFIG *gddc)
{
        static unsigned char    packet[CHUNKSIZE];
        static unsigned char    tgtbuf[CHUNKSIZE+2];
        SOCKET                  sock;                         
        int                     l,n;
        unsigned char           *buffer_p=NULL,*p;
        struct bpf_hdr          *nit_buffer_header;
        int                     csize;

        if (init_connection_table()){
                errorlog(ERRLOG_FATAL,"Can not initialize contable\n");
                return;
        }
        if ((sock=setnic_promisc(gddc->nit_dev,gddc->nic_name,&csize))<0){
                errorlog(ERRLOG_FATAL,"setnic_promisc error (%d)\n",sock);
                release_connection_table();
                return;
        }
        if (sizeof(packet)<csize){
                errorlog(ERRLOG_FATAL,"chunksize is too big (%d)\n",csize);
                release_connection_table();
        }
        for (;;){
                if ((l=read(sock,packet,csize))<0){
                        errorlog(ERRLOG_FATAL,"read error(%d)\n",l);
                        release_connection_table();
                        return;
                }
                if (l>0){
                        buffer_p=(unsigned char *)packet;
                        while (buffer_p < packet+l) {
                                nit_buffer_header=(struct bpf_hdr *)buffer_p;
                                p=buffer_p+nit_buffer_header->bh_hdrlen;
                                n=nit_buffer_header->bh_caplen;
                                memcpy(tgtbuf+2,p,n);  /* For RISC CPU */
                                if (packet_analysis(gddc,tgtbuf+2,n)){
                                        release_connection_table();
                                        return;
                                }
                                n=nit_buffer_header->bh_caplen
                                 +nit_buffer_header->bh_hdrlen;
                                buffer_p += BPF_WORDALIGN(n);
                        }
                }
        }
}
